from typing import List
from google.oauth2.credentials import Credentials
from google_auth_oauthlib.flow import InstalledAppFlow
from googleapiclient.discovery import build
from googleapiclient.errors import HttpError
from Extensions import Extensions


class microsoft_365(Extensions):
    def __init__(
        self,
        MICROSOFT_365_CLIENT_ID: str = "",
        MICROSOFT_365_CLIENT_SECRET: str = "",
        MICROSOFT_365_REDIRECT_URI: str = "",
        **kwargs,
    ):
        self.MICROSOFT_365_CLIENT_ID = MICROSOFT_365_CLIENT_ID
        self.MICROSOFT_365_CLIENT_SECRET = MICROSOFT_365_CLIENT_SECRET
        self.MICROSOFT_365_REDIRECT_URI = MICROSOFT_365_REDIRECT_URI
        if (
            self.MICROSOFT_365_CLIENT_ID
            and self.MICROSOFT_365_CLIENT_SECRET
            and self.MICROSOFT_365_REDIRECT_URI
        ):
            self.commands = {
                "Send Email with Microsoft 365": self.send_email,
                "Check Email with Microsoft 365": self.check_email,
                "Move Email with Microsoft 365": self.move_email,
            }
            try:
                if (
                    self.MICROSOFT_365_CLIENT_SECRET != "None"
                    and self.MICROSOFT_365_CLIENT_SECRET != ""
                ):
                    self.credentials = self.get_credentials()
            except:
                pass

    async def get_credentials(self) -> Credentials:
        flow = InstalledAppFlow.from_client_config(
            {
                "installed": {
                    "client_id": self.MICROSOFT_365_CLIENT_ID,
                    "client_secret": self.MICROSOFT_365_CLIENT_SECRET,
                    "redirect_uris": [self.MICROSOFT_365_REDIRECT_URI],
                    "auth_uri": "https://login.microsoftonline.com/common/oauth2/v2.0/authorize",
                    "token_uri": "https://login.microsoftonline.com/common/oauth2/v2.0/token",
                }
            },
            ["https://graph.microsoft.com/Mail.ReadWrite"],
        )

        return flow.run_local_server(port=0)

    async def send_email(
        self, from_email: str, to_email: str, subject: str, content: str
    ) -> List[str]:
        try:
            message = {
                "message": {
                    "subject": subject,
                    "body": {"contentType": "HTML", "content": content},
                    "toRecipients": [{"emailAddress": {"address": to_email}}],
                    "from": {"emailAddress": {"address": from_email}},
                },
                "saveToSentItems": "true",
            }

            service = build("graph.microsoft.com", "v1.0", credentials=self.credentials)
            response = (
                service.users().messages().send(userId="me", body=message).execute()
            )
            return [f"Email sent successfully. Message ID: {response['id']}"]

        except HttpError as error:
            return [f"Error sending email: {error}"]

    async def check_email(self) -> List[str]:
        try:
            service = build("graph.microsoft.com", "v1.0", credentials=self.credentials)
            response = service.users().messages().list(userId="me").execute()
            emails = response.get("value", [])

            result = []
            for email in emails:
                result.append(f"Email ID: {email['id']}, Subject: {email['subject']}")

            return result

        except HttpError as error:
            return [f"Error checking email: {error}"]

    async def move_email(
        self, message_id: str, destination_folder_id: str
    ) -> List[str]:
        try:
            service = build("graph.microsoft.com", "v1.0", credentials=self.credentials)
            response = (
                service.users()
                .messages()
                .move(
                    userId="me",
                    id=message_id,
                    body={"destinationId": destination_folder_id},
                )
                .execute()
            )
            return [
                f"Email moved successfully. New folder ID: {response['parentFolderId']}"
            ]

        except HttpError as error:
            return [f"Error moving email: {error}"]



from typing import List
from sendgrid import SendGridAPIClient
from sendgrid.helpers.mail import Mail
from Extensions import Extensions


class sendgrid_email(Extensions):
    def __init__(self, SENDGRID_API_KEY: str = "", SENDGRID_EMAIL: str = "", **kwargs):
        self.SENDGRID_API_KEY = SENDGRID_API_KEY
        self.SENDGRID_EMAIL = SENDGRID_EMAIL
        if self.SENDGRID_API_KEY:
            self.commands = {"Send Email with Sendgrid": self.send_email}

    async def send_email(self, to_email: str, subject: str, content: str) -> List[str]:
        message = Mail(
            from_email=self.SENDGRID_EMAIL,
            to_emails=to_email,
            subject=subject,
            html_content=content,
        )

        try:
            sg = SendGridAPIClient(self.SENDGRID_API_KEY)
            response = sg.send(message)
            return [f"Email sent successfully. Status code: {response.status_code}"]
        except Exception as e:
            return [f"Error sending email: {e}"]



import os
import json
from AGiXT import AGiXT
import argparse
from Prompts import Prompts
from Extensions import Extensions
import logging


class Chain:
    def get_chain(self, chain_name):
        with open(os.path.join("chains", f"{chain_name}.json"), "r") as f:
            chain_data = json.load(f)
        return chain_data

    def get_chains(self):
        chains = [
            f.replace(".json", "") for f in os.listdir("chains") if f.endswith(".json")
        ]
        return chains

    def add_chain(self, chain_name):
        chain_data = {"chain_name": chain_name, "steps": []}
        with open(os.path.join("chains", f"{chain_name}.json"), "w") as f:
            json.dump(chain_data, f)

    def rename_chain(self, chain_name, new_name):
        os.rename(
            os.path.join("chains", f"{chain_name}.json"),
            os.path.join("chains", f"{new_name}.json"),
        )

    def add_chain_step(self, chain_name, step_number, agent_name, prompt_type, prompt):
        chain_data = self.get_chain(chain_name=chain_name)
        chain_data["steps"].append(
            {
                "step": step_number,
                "agent_name": agent_name,
                "prompt_type": prompt_type,
                "prompt": prompt,
            }
        )
        with open(os.path.join("chains", f"{chain_name}.json"), "w") as f:
            json.dump(chain_data, f)

    def update_step(self, chain_name, step_number, agent_name, prompt_type, prompt):
        chain_data = self.get_chain(chain_name=chain_name)
        for step in chain_data["steps"]:
            if step["step"] == step_number:
                step["agent_name"] = agent_name
                step["prompt_type"] = prompt_type
                step["prompt"] = prompt
                break
        with open(os.path.join("chains", f"{chain_name}.json"), "w") as f:
            json.dump(chain_data, f)

    def delete_step(self, chain_name, step_number):
        chain_data = self.get_chain(chain_name=chain_name)
        chain_data["steps"] = [
            step for step in chain_data["steps"] if step["step"] != step_number
        ]
        with open(os.path.join("chains", f"{chain_name}.json"), "w") as f:
            json.dump(chain_data, f)

    def delete_chain(self, chain_name):
        os.remove(os.path.join("chains", f"{chain_name}.json"))

    def get_step(self, chain_name, step_number):
        chain_data = self.get_chain(chain_name=chain_name)
        for step in chain_data["steps"]:
            if step["step"] == step_number:
                return step
        return None

    def get_steps(self, chain_name):
        chain_data = self.get_chain(chain_name=chain_name)
        return chain_data["steps"]

    async def run_chain(self, chain_name):
        chain_data = self.get_chain(chain_name=chain_name)
        logging.info(f"Running chain '{chain_name}'")
        responses = {}  # Create a dictionary to hold responses.
        for step_data in chain_data["steps"]:
            if "prompt" in step_data and "step" in step_data:
                logging.info(f"Running step {step_data['step']}")
                step_response = await self.run_chain_step(
                    step=step_data, chain_name=chain_name
                )  # Get the response of the current step.
                responses[step_data["step"]] = step_response  # Store the response.
                logging.info(f"Response: {step_response}")
        # Write the responses to the json file.
        with open(os.path.join("chains", f"{chain_name}_responses.json"), "w") as f:
            json.dump(responses, f)
        return responses

    def get_step_response(self, chain_name, step_number="all"):
        try:
            with open(os.path.join("chains", f"{chain_name}.json"), "r") as f:
                responses = json.load(f)
            if step_number == "all":
                return responses
            else:
                return responses.get(str(step_number))
        except:
            return ""

    def get_step_content(self, chain_name, step_number, prompt_content):
        if "{STEP" in prompt_content:
            # get the step number from the prompt content
            step_number = int(
                prompt_content[
                    prompt_content.find("{STEP") + 5 : prompt_content.find("}")
                ]
            )
            # get the response from the step number
            step_response = self.get_step_response(
                chain_name=chain_name, step_number=step_number
            )
            # replace the {STEPx} with the response
            prompt_content = prompt_content.replace(
                f"{{STEP{step_number}}}", step_response
            )
        return prompt_content

    async def run_chain_step(self, step: dict = {}, chain_name=""):
        logging.info(step)
        if step:
            if "prompt_type" in step:
                prompt_type = step["prompt_type"]
                prompt = step["prompt"]
                agent_name = step["agent_name"]
                agent = AGiXT(agent_name)
                step_number = step["step"]
                try:
                    command_name = prompt["command_name"]
                except:
                    command_name = ""
                if prompt_type == "Command":
                    commands_args = prompt.copy()
                    for arg in commands_args:
                        commands_args[arg] = self.get_step_content(
                            chain_name, step_number, commands_args[arg]
                        )
                    return await Extensions(
                        agent_config=agent.agent.agent_config
                    ).execute_command(
                        command_name=command_name, command_args=commands_args
                    )
                try:
                    prompt_content = Prompts().get_prompt(
                        prompt_name=prompt["prompt_name"]
                    )
                    prompt_content = self.get_step_content(
                        chain_name, step_number, prompt_content
                    )
                except:
                    return None
                if prompt_type == "Prompt":
                    result = await agent.run(
                        prompt=prompt["prompt_name"],
                        chain_name=chain_name,
                        step_number=step_number,
                        **prompt,
                    )
                elif prompt_type == "Chain":
                    result = await self.run_chain(prompt["chain_name"])
                elif prompt_type == "Smart Instruct":
                    result = await agent.smart_instruct(task=prompt_content, **prompt)
                elif prompt_type == "Smart Chat":
                    result = await agent.smart_chat(task=prompt_content, **prompt)
                elif prompt_type == "Task":
                    result = await agent.run_task(objective=prompt_content, **prompt)
        if result:
            return result
        else:
            return None


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("--chain", type=str, default="")
    args = parser.parse_args()
    chain_name = args.chain
    import asyncio

    asyncio.run(Chain().run_chain(chain_name=chain_name))



from datetime import datetime
from Extensions import Extensions


class times(Extensions):
    def __init__(self, **kwargs):
        self.commands = {"Get Datetime": self.get_datetime}

    async def get_datetime(self) -> str:
        return "Current date and time: " + datetime.now().strftime("%Y-%m-%d %H:%M:%S")



import streamlit as st
import bcrypt
import time
from auth_libs.Cfig import Cfig
from auth_libs.Users import (
    load_users,
    save_user_data,
    configure_auth_settings,
    check_admin_configured,
)
from auth_libs.Users import check_auth_status

logged = st.session_state.get("logged_in")
if logged:
    st.session_state["login_page"] = True
check_auth_status()
CFIG = Cfig()


# Login form
def login_form():
    """
    Renders the login form.
    """
    if CFIG.load_config()["auth_setup_config"] == "No Login":
        st.write("Login Is Not Enabled")
        st.stop()

    user_data = load_users()
    if user_data == "Reload":
        st.experimental_rerun()

    st.write("Please log in")
    email = st.text_input("Email")
    password = st.text_input("Password", type="password")

    if st.button("Login"):
        for user in user_data["users"]:
            if user["email"] == email:
                # st.write("Email: " + str(user["email"]))
                # st.write(" Input Pass: " + str(bcrypt.hashpw(password.encode(),user["password_hash"].encode(),)))
                # st.write(" Saved Pass: " + str(user["password_hash"].encode("utf-8")))
                if bcrypt.checkpw(
                    password.encode("utf-8"), user["password_hash"].encode("utf-8")
                ):
                    st.success("Login successful!")
                    st.session_state["logged_in"] = True
                    st.session_state["email"] = email
                    time.sleep(1)
                    st.experimental_rerun()  # Redirect to UI
        else:
            st.error("Incorrect email or password.")
        st.stop()


# Check if admin configuration is needed
if not check_admin_configured():
    configure_auth_settings()
    st.stop()

# Render the login form
login_form()
